    /*********************************************************************\
    *  Copyright (c) 1991 by Wen-King Su (wen-king@vlsi.cs.caltech.edu)   *
    *                                                                     *
    *  You may copy or modify this file in any manner you wish, provided  *
    *  that this notice is always included, and that you hold the author  *
    *  harmless for any loss or damage resulting from the installation or *
    *  use of this software.                                              *
    \*********************************************************************/

#include "tweak.h"
#include "client_def.h"
#include "c_extern.h"
#include "bsd_extern.h"
#include "co_extern.h"
#include <signal.h>
#ifdef STDC_HEADERS
#include <stdlib.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#include "my-string.h"

int get_clobbertype=C_TEMPNAME;
int preserve=0;
char *fname;
int optletter;
int suffix;
char *tname;
unsigned long start_from;

static int len;

static RETSIGTYPE fsp_cleanup PROTO1(int, signum)
{
  char filename[20];
  sprintf(filename,".fsp.%d",getpid());
  unlink(filename);
  exit(1);
}

static void get_file PROTO4(char *, path, struct stat *, sbufp, int, mode, int, level)
{
  char *name = path + len;
  FILE *fp;
  struct stat statbuf;
  
  start_from = 0; 
  if (get_clobbertype==C_NOCLOBBER) {
    if ( (fp=fopen(name,"r"))) {
      fclose(fp);
      fprintf(stderr,"Will not overwrite existing file %s\n",name);
      return;
    }
  }
  
  if (get_clobbertype==C_UNIQUE) {
    fname=name;
    name=(char *)malloc(strlen(fname)+5);
    strcpy(name,fname);
    for (suffix=0 ; (fp=fopen(name,"r")) ; suffix++) {
      fclose(fp);
      sprintf(name,"%s-%d",fname,suffix);
    }
  }
  
  if (get_clobbertype==C_TEMPNAME) {
    fname=name;
    name=(char*)malloc(20);
    sprintf(name,".fsp.%d",getpid());
  }
  
  if(get_clobbertype == C_APPEND)  {
    if(stat(name, &statbuf) == 0) {
      start_from = statbuf.st_size;
      if(start_from != 0)
         if((fp = fopen(name,"a")) == NULL) { perror(name);
                                             start_from = 0;}
    } else start_from = 0;
  } else start_from = 0;
  
  if(start_from == 0) {
    fp = fopen(name, "w");
    if (fp == NULL )
            fprintf(stderr,"Cannot create %s\n",name);
  }
  
  if(fp) {
    if(util_download(path,fp, start_from) == -1) {
      fclose(fp);
      unlink(name);
    } else fclose(fp);
  } else fprintf(stderr,"Cannot write %s\n",name);
  
  if (get_clobbertype==C_TEMPNAME) {
    rename(name,fname);
    free(name);
  }
  
  if (get_clobbertype==C_UNIQUE) {
    free(name);
  }
  return;
}

static int make_dir PROTO3(char *, name, struct stat *, sbufp, u_long *, mode)
{
    struct stat sbuf;

    if (*mode == 0) return (-1);

    if (stat(name + len, &sbuf) == 0)
    {
        /* check if the directory already exists... */
        if (S_ISDIR(sbuf.st_mode))
            return (0);

        /* the directory doesn't exist, but *something* does... urgh! */
        fprintf(stderr,"fgetcmd: local file `%s' is not a directory\n",
                 name + len);
        return (-1);
    }

    /* nothing exists by this name -- try to create it */
    if (mkdir(name + len, 0755) < 0)
    {
        perror (name + len);
        return(-1);
    }

    return (0);
}

int main PROTO2(int, argc, char **, argv)
{
  char **av, *av2[2], n[1024];
  int prompt, mode = 0;
  
  signal(SIGHUP,fsp_cleanup);
  signal(SIGINT,fsp_cleanup);
  signal(SIGQUIT,fsp_cleanup);
  signal(SIGILL,fsp_cleanup);
  signal(SIGTRAP,fsp_cleanup);
  signal(SIGFPE,fsp_cleanup);
  signal(SIGSEGV,fsp_cleanup);
#ifndef __linux__
  signal(SIGEMT,fsp_cleanup);
  signal(SIGBUS,fsp_cleanup);
  signal(SIGSYS,fsp_cleanup);
#endif
  signal(SIGPIPE,fsp_cleanup);
  signal(SIGTERM,fsp_cleanup);

  env_client();
  if (strcmp(env_local_dir,".") && chdir(env_local_dir)) {
    perror("chdir");
    exit(1);
  }
  
  /* Parse options
   *   -f forces overwrite
   *   -u forces unique names
   *   -t uses temporary file to download
   *   -n forces noclobber
   *   -a append to files if they exist
   *   -c same as -a
   *   -r recursively get directories 
   *   -p preserve date/times of original file on downloaded copy
   */

  while ((optletter=getopt(argc, argv,"futnacrp")) != EOF)
    switch (optletter) {
      case 'f':
        get_clobbertype=C_CLOBBER;
	break;
      case 'u':
	get_clobbertype=C_UNIQUE;
	break;
      case 't':
	get_clobbertype=C_TEMPNAME;
	break;
      case 'n':
	get_clobbertype=C_NOCLOBBER;
	break;
      case 'a':
      case 'c':
 	get_clobbertype = C_APPEND;
	break;
      case 'r':
	mode=1;
	break;
      case 'p':
	preserve=1;
	break;
    }
  
  if(argc > optind) {
    for( ; argc>optind ; optind++) {
      if(!(av = glob(argv[optind])))  {
	av = av2;
	av2[0] = argv[optind];
	av2[1] = 0;
      }
      while(*av)
      {
         for(len = strlen(*av); len >= 0 && (*av)[len] != '/'; len--);
         len++;
         util_process_file(*av, mode, get_file, make_dir, 0L, 0);
         av++;
      }

    }
  } else {
    prompt = isatty(0);
    while(1) {
      if(prompt) {
	fputs("fget: ",stdout);
	fflush(stdout);
      }
      if(!getsl(n,1024)) break;
      if(!*n) continue;
      if(!(av = glob(n))) {
	av = av2;
	av2[0] = n;
	av2[1] = 0;
      }
      while(*av)
      {
         for(len = strlen(*av); len >= 0 && (*av)[len] != '/'; len--);
         len++;
         util_process_file(*av, mode, get_file, make_dir, 0L, 0);
         av++;
      }
    }
  }
  
  client_done();
  
  exit(0);
}
